"use strict";
var HEX_SHORT = /^#([a-fA-F0-9]{3})$/;
var HEX = /^#([a-fA-F0-9]{6})$/;
function roundColors(colors, round) {
    if (!round)
        return colors;
    var output = {};
    for (var key in colors) {
        output[key] = Math.round(colors[key]);
    }
    return output;
}
function hasProp(obj, key) {
    return obj.hasOwnProperty(key);
}
function isRgb(obj) {
    return hasProp(obj, "red") && hasProp(obj, "green") && hasProp(obj, "blue");
}
var Color = (function () {
    function Color(value) {
        this._original = value;
        if (isRgb(value)) {
            this._rgb = value;
            this._hex = Color.rgbToHex(value);
        }
        else {
            this._hex = Color.normalizeHex(value);
            this._rgb = Color.hexToRgb(this._hex);
        }
        this._hsv = Color.rgbToHsv(this._rgb);
    }
    Color.normalizeHex = function (hex) {
        if (HEX.test(hex)) {
            return hex;
        }
        else if (HEX_SHORT.test(hex)) {
            var r = hex.slice(1, 2);
            var g = hex.slice(2, 3);
            var b = hex.slice(3, 4);
            return "#" + (r + r) + (g + g) + (b + b);
        }
        return null;
    };
    Color.hexToRgb = function (hex) {
        var normalizedHex = this.normalizeHex(hex);
        if (normalizedHex == null) {
            return null;
        }
        var m = normalizedHex.match(HEX);
        var i = parseInt(m[1], 16);
        var red = (i >> 16) & 0xFF;
        var green = (i >> 8) & 0xFF;
        var blue = i & 0xFF;
        return { red: red, green: green, blue: blue };
    };
    Color.rgbToHex = function (rgb) {
        var red = rgb.red, green = rgb.green, blue = rgb.blue;
        var i = ((Math.round(red) & 0xFF) << 16) + ((Math.round(green) & 0xFF) << 8) + (Math.round(blue) & 0xFF);
        var s = i.toString(16).toLowerCase();
        return "#" + ("000000".substring(s.length) + s);
    };
    Color.rgbToHsv = function (rgb, round) {
        if (round === void 0) { round = true; }
        var red = rgb.red, green = rgb.green, blue = rgb.blue;
        var min = Math.min(red, green, blue);
        var max = Math.max(red, green, blue);
        var delta = max - min;
        var hsv = {};
        if (max === 0) {
            hsv.saturation = 0;
        }
        else {
            hsv.saturation = (delta / max * 1000) / 10;
        }
        if (max === min) {
            hsv.hue = 0;
        }
        else if (red === max) {
            hsv.hue = (green - blue) / delta;
        }
        else if (green === max) {
            hsv.hue = 2 + (blue - red) / delta;
        }
        else {
            hsv.hue = 4 + (red - green) / delta;
        }
        hsv.hue = Math.min(hsv.hue * 60, 360);
        hsv.hue = hsv.hue < 0 ? hsv.hue + 360 : hsv.hue;
        hsv.value = ((max / 255) * 1000) / 10;
        return roundColors(hsv, round);
    };
    Color.rgbToXyz = function (rgb, round) {
        if (round === void 0) { round = true; }
        var red = rgb.red / 255;
        var green = rgb.green / 255;
        var blue = rgb.blue / 255;
        var rr = red > 0.04045 ? Math.pow(((red + 0.055) / 1.055), 2.4) : red / 12.92;
        var gg = green > 0.04045 ? Math.pow(((green + 0.055) / 1.055), 2.4) : green / 12.92;
        var bb = blue > 0.04045 ? Math.pow(((blue + 0.055) / 1.055), 2.4) : blue / 12.92;
        var x = (rr * 0.4124 + gg * 0.3576 + bb * 0.1805) * 100;
        var y = (rr * 0.2126 + gg * 0.7152 + bb * 0.0722) * 100;
        var z = (rr * 0.0193 + gg * 0.1192 + bb * 0.9505) * 100;
        var xyz = { x: x, y: y, z: z };
        return roundColors(xyz, round);
    };
    Color.rgbToLab = function (rgb, round) {
        if (round === void 0) { round = true; }
        var xyz = Color.rgbToXyz(rgb, false);
        var x = xyz.x, y = xyz.y, z = xyz.z;
        x /= 95.047;
        y /= 100;
        z /= 108.883;
        x = x > 0.008856 ? Math.pow(x, 1 / 3) : 7.787 * x + 16 / 116;
        y = y > 0.008856 ? Math.pow(y, 1 / 3) : 7.787 * y + 16 / 116;
        z = z > 0.008856 ? Math.pow(z, 1 / 3) : 7.787 * z + 16 / 116;
        var l = (116 * y) - 16;
        var a = 500 * (x - y);
        var b = 200 * (y - z);
        return roundColors({ l: l, a: a, b: b }, round);
    };
    Object.defineProperty(Color.prototype, "original", {
        get: function () {
            return this._original;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Color.prototype, "rgb", {
        get: function () {
            return this._rgb;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Color.prototype, "hex", {
        get: function () {
            return this._hex;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Color.prototype, "hsv", {
        get: function () {
            return this._hsv;
        },
        enumerable: true,
        configurable: true
    });
    return Color;
}());
exports.Color = Color;
//# sourceMappingURL=color.js.map